/**
 * Class for a simulation of a crowd
 * @author Cassy Romano
 * @author Barb Ericson
 */
public class CrowdAgent extends Agent {
  
  /** the pictures to use */
  private static Picture dog1, dog2, dogwag;
  
  ////////////////////////////// Constructors ////////////////////////
  
  /**
   * Constructor that takes the model display (the original
   * position will be randomly assigned) and the simulation
   * @param modelDisplayer thing that displays the model
   * @param thisSim my simulation
   */
  public CrowdAgent (ModelDisplay modelDisplayer,Simulation thisSim) {
    super(randNumGen.nextInt(modelDisplayer.getWidth()),
          randNumGen.nextInt(modelDisplayer.getHeight()),
          modelDisplayer, thisSim);
  }
  
  /** Constructor that takes the x and y, a model
    * display to draw it on, and a simulation
    * @param x the starting x position
    * @param y the starting y position
    * @param modelDisplayer the thing that displays the model
    * @param thisSim my simulation
    */
  public CrowdAgent (int x, int y, 
                     ModelDisplay modelDisplayer, 
                     Simulation thisSim) {
    // let the parent constructor handle it
    super(x,y,modelDisplayer,thisSim);
  }
  
  //////////////////// methods /////////////////////////////////
  
  /**
   * Set up the simulation
   * @param thisSim the simulation to use
   */
  public void init(Simulation thisSim) {
    if (dog1 == null) {
      
      // Do we have the dog characters defined yet?
      dog1 = new Picture(FileChooser.getMediaPath("mydog.jpg")).scale(0.5);
      dog2 = new Picture(FileChooser.getMediaPath("mydog2.jpg")).scale(0.5);
      dogwag = new Picture(FileChooser.getMediaPath("mydog-wag.jpg")).scale(0.5);
      
      // Start out with myPict as dog1
      this.setPicture(dog1);
      
      // Do the normal initializations
      super.init(thisSim);
      
      // Place all dogs randomly within world
      this.setPenDown(false);
      this.moveTo(randNumGen.nextInt(600), randNumGen.nextInt(600));
      
      // Set speed to relatively slow
      this.setSpeed(5);
      
    }
  }
  
  /**
   * act(t) For first 10 steps, walk around,
   * Then walk AWAY from the bigger, mighter dog "Hunter", 
   * and coninue wandering around.
   * @param t the current timestep
   */
  public void act(int t) {
    
    // Tell it that this really is a CrowdSimulation
    CrowdSimulation mySim = (CrowdSimulation) getSimulation();
    this.setPicture(dog1); 
    
    if (t >= 10) {
      this.setPicture(dog1);
      if (this.getDistance(mySim.getHunter().getXPos(), 
                           mySim.getHunter().getYPos()) <= 30) {
        this.turnToFace(mySim.getHunter());
        this.turn(180);
        this.forward(40);
      }
      else if (this.getDistance(mySim.getHunter().getXPos(), 
                                mySim.getHunter().getYPos()) > 30) {
        this.forward(randNumGen.nextInt(5));
      }
    }
    
    // First, handle motion
    else if (t < 10) {
      if (0 == this.countInRange(50, mySim.getAgents())) {
        Agent closestAgent = this.getClosest(50, mySim.getAgents());
        this.turnToFace(closestAgent);
        this.forward(2);
      }
      if (5 == this.countInRange(10, mySim.getAgents())) {
        Agent closestAgent2 = this.getClosest(10, mySim.getAgents());
        this.turnToFace(closestAgent2);
        this.turn(180);
        this.forward(15);
      }
      if (1 == randNumGen.nextInt(10)) { 
        this.turn(randNumGen.nextInt(80)-40); 
        this.forward(randNumGen.nextInt(5));
      }
      if (3 == randNumGen.nextInt(12)) { 
        this.setPicture(dog2); 
        this.forward(randNumGen.nextInt(5)); 
      }
      if (1 == randNumGen.nextInt(20)) { 
        this.setPicture(dogwag); 
        this.forward(randNumGen.nextInt(5));
      }
    } 
  }// end act
  
}


