/**
 * WebPageReader class
 * Given a URL, can return information about that page.
 * @author Mark Guzdial
 * @author Barb Ericson
 */
import java.net.*;
import java.io.*;
import java.util.*;

public class WebPageReader {
  
  /////////   Fields ///////////////////////////
  
  /** the url to read from */
  private URL url;
  
  /** the url connection to use */
  private URLConnection con;
  
  /** the buffered reader to use */
  private BufferedReader stream;
  
  ///// Methods ////////////////////
  
  /**
   * Constructor that takes a url as a string
   * @param s the url as a string
   */
  public WebPageReader(String s) {
    // Create the URL and the connection to it
    try {
      url = new URL(s);
      con = url.openConnection();
      stream = new BufferedReader(new InputStreamReader(con.getInputStream()));
    } catch (Exception e) {
      System.out.println("An error opening the URL occurred.");
      System.out.println(e.getMessage());
    }
  }
  
  /**
   * A WebPageReader is ready to read if the stream is ready
   * @return true if the stream is ready, else false
   */
  public boolean readyToRead() {
    try {
      return stream.ready();
    }
    catch (Exception e) {System.out.println("I/O error occurred.");
      System.out.println(e.getMessage());
      return false;
    }
  }
  
  /**
   * The type of the material at the other end of the URL is
   * the contentType from the URLConnection
   * @return a string with the content type
   */
  public String getType() {return con.getContentType();}
  
  /**
   * Next line gets the next line from the material at the
   * other end of the URL.  We read it like a file.
   * There is more material there as long as readyToRead() returns
   * true.  We may also read a null when it's done.
   * @return the next line from the url or null if done
   */
  public String nextLine() {
    try {
      return stream.readLine();
    } catch (Exception e) {System.out.println("I/O error occurred.");
      System.out.println(e.getMessage());
      return null;
    }
  }
  
}
